package App::Yath::Schema::Result::Email;
use utf8;
use strict;
use warnings;

our $VERSION = '2.000003'; # TRIAL

# DO NOT MODIFY THIS FILE, GENERATED BY author_tools/regen_schema.pl


use Carp qw/confess/;
confess "You must first load a App::Yath::Schema::NAME module"
    unless $App::Yath::Schema::LOADED;

require "App/Yath/Schema/${App::Yath::Schema::LOADED}/Email.pm";
require "App/Yath/Schema/Overlay/Email.pm";

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

App::Yath::Schema::Result::Email - Autogenerated result class for Email.

=head1 SEE ALSO

L<App::Yath::Schema::Overlay::Email> - Where methods that are not
auto-generated are defined.

=head1 SOURCE

The source code repository for Test2-Harness can be found at
L<http://github.com/Test-More/Test2-Harness/>.

=head1 MAINTAINERS

=over 4

=item Chad Granum E<lt>exodist@cpan.orgE<gt>

=back

=head1 AUTHORS

=over 4

=item Chad Granum E<lt>exodist@cpan.orgE<gt>

=back

=head1 COPYRIGHT

Copyright Chad Granum E<lt>exodist7@gmail.comE<gt>.

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

See L<http://dev.perl.org/licenses/>

=cut
